#!/usr/bin/env python
# -*- coding: utf-8 -*-

ROOT_AGRO_API = 'agromonitoring.com/agro/1.0'
ROOT_DOWNLOAD_PNG_API = 'agromonitoring.com/image/1.0'
ROOT_DOWNLOAD_GEOTIFF_API = 'agromonitoring.com/data/1.0'

# Polygons API subset
POLYGONS_URI = 'polygons'
NAMED_POLYGON_URI = 'polygons/%s'

# Soil API subset
SOIL_URI = 'soil'

# Satellite Imagery Search API subset
SATELLITE_IMAGERY_SEARCH_URI = 'image/search'
