#!/usr/bin/env python
# -*- coding: utf-8 -*-


class MapLayerEnum:
    """
    Allowed map layer values for tiles retrieval

    """
    PRECIPITATION = 'precipitation_new'
    WIND = 'wind_new'
    TEMPERATURE = 'temp_new'
    PRESSURE = 'pressure_new'
