#!/usr/bin/env python
# -*- coding: utf-8 -*-

AGRO_API_VERSION = (1, 0, 0)
AIRPOLLUTION_API_VERSION = (3, 0, 0)
ALERT_API_VERSION = (3, 0, 0)
GEOCODING_API_VERSION = (1, 0, 0)
STATIONS_API_VERSION = (3, 0, 0)
UVINDEX_API_VERSION = (3, 0, 0)
WEATHER_API_VERSION = (2, 5, 0)
LANGUAGES = [
    "af",
    "al",
    "ar",
    "az",
    "bg",
    "ca",
    "cz",
    "da",
    "de",
    "el",
    "en",
    "es",
    "eu",
    "fa",
    "fi",
    "fr",
    "gl",
    "he",
    "hi",
    "hr",
    "hu",
    "id",
    "it",
    "ja",
    "kr",
    "la",
    "lt",
    "mk",
    "nl",
    "no",
    "pl",
    "pt",
    "pt_br",
    "ro",
    "ru",
    "se",
    "sk",
    "sl",
    "sp",
    "sr",
    "sv",
    "th",
    "tr",
    "ua",
    "uk",
    "vi",
    "zh_cn",
    "zh_tw",
    "zu",
]