#!/usr/bin/env python
# -*- coding: utf-8 -*-

# deprecated API endpoints
ROOT_POLLUTION_API_URL = 'openweathermap.org/pollution/v1'
CO_INDEX_URL = 'co'
OZONE_URL = 'o3'
NO2_INDEX_URL = 'no2'
SO2_INDEX_URL = 'so2'


# current API endpoint
NEW_ROOT_POLLUTION_API_URL = 'openweathermap.org/data/2.5'
AIR_POLLUTION_URL = 'air_pollution'
AIR_POLLUTION_FORECAST_URL = 'air_pollution/forecast'
AIR_POLLUTION_HISTORY_URL = 'air_pollution/history'
