#!/usr/bin/env python
# -*- coding: utf-8 -*-

ROOT_ALERT_API_URL = 'openweathermap.org/data/3.0'

# Triggers
TRIGGERS_URI = 'triggers'
NAMED_TRIGGER_URI = 'triggers/%s'

# Alerts
ALERTS_URI = 'triggers/%s/history'
NAMED_ALERT_URI = 'triggers/%s/history/%s'
